/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebMail Notifier.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{e3ab0980-b4e1-11db-abbd-0800200c9a66}");

// description
const CLASS_NAME = "WebMail Notifier Service";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/WebMailNotifier;1";
const ADDON_ID = "{37fa1426-b82d-11db-8314-0800200c9a66}";

const Ci=Components.interfaces;

const ST_CHECK       = 0;
const ST_PRE         = 10;
const ST_PRE_RES     = 11;
const ST_LOGIN       = 20;
const ST_LOGIN_RES   = 21;
const ST_DATA        = 30;
const ST_DATA_RES    = 31;

const LOGIN_URL      = 0;
const LOGIN_ID       = 1;
const LOGIN_PW       = 2;
const LOGIN_DATA     = 3;
const LOGIN_POST     = 1;

const formSubmitURL = "user";

const dout=Components.utils.reportError;

var enableDebug=0;
/***********************************************************
class definition
***********************************************************/

//class constructor
function WebMailNotifier() {
  this.wrappedJSObject = this;
  var observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Ci.nsIObserverService);
  observerService.addObserver(this, "http-on-modify-request", false);
  observerService.addObserver(this, "http-on-examine-response", false);
  //for uninstall
  this.uninstall=false;
  observerService.addObserver(this, "em-action-requested", false);
  observerService.addObserver(this, "quit-application-granted", false);

  //init preferences
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Ci.nsIPrefService);
  this.prefBranch = prefService.getBranch("extensions.wmn.");
  this.prefBranch.QueryInterface(Ci.nsIPrefBranch2);
  this.prefBranch.addObserver("", this, false);
  this.timerDelay=this.prefBranch.getIntPref("updateInterval");
  this.connectionDelay=this.prefBranch.getIntPref("connectionDelay")*1000;
  this.resetCounter=this.prefBranch.getBoolPref("resetCounter");
  this.multiSession=this.prefBranch.getBoolPref("multiSession");
  
  try{
    var ss = Components.classes["@mozilla.org/browser/sessionstore;1"]
                      .getService(Components.interfaces.nsISessionStore);
    ss.persistTabAttribute("wmntab");
    ss.persistTabAttribute("wmnsid");
    ss.persistTabAttribute("wmnid");
  }catch(e){}
  var tmp=this;
  this.worker={
    notify:function(aTimer){
      WebMailNotifier.prototype.timerWork.call(tmp,aTimer);
    }
  };
  this.timer = Components.classes["@mozilla.org/timer;1"]
                .createInstance(Ci.nsITimer);
  this.timer.initWithCallback(this,
                                  60000,
                                  Ci.nsITimer.TYPE_REPEATING_PRECISE);

  this.workTimer = Components.classes["@mozilla.org/timer;1"]
                .createInstance(Ci.nsITimer);


  enableDebug=this.prefBranch.getIntPref("debug");

  if ("@mozilla.org/login-manager;1" in Components.classes){
    this.loginManager = Components.classes["@mozilla.org/login-manager;1"]
                           .getService(Ci.nsILoginManager);
  }else if ("@mozilla.org/passwordmanager;1" in Components.classes){
    this.passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                  .getService(Ci.nsIPasswordManager);
  }
  if("@mozilla.org/browser/favicon-service;1" in Components.classes){
    this.faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
                       .getService(Ci.nsIFaviconService);  
  }
  this.init();//init before session restore;
}
// class definition
WebMailNotifier.prototype = {
  MSG_MAILDATA:0,
  MSG_INIT:-1,
  MSG_RESET:-2,
  MSG_SHOWICON:-3,
  MSG_SHOWTBBTN:-4,
  MSG_ENABLED:-5,

  appName: "WebMail Notifier",
  classDescription: CLASS_NAME,
  classID: CLASS_ID,
  contractID: CONTRACT_ID,
  listeners : new Array(),
  timer : null,
  initCalled : false,
  countTotal : 0,
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(Ci.nsIWebProgressListener)&&
        !aIID.equals(Ci.nsISupportsWeakReference) &&
        !aIID.equals(Ci.nsISupports)){
      throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  }
}
WebMailNotifier.prototype.init2=function() {
  if (typeof AddonManager != "undefined"){
    var tmp=this;
    AddonManager.getAddonByID(ADDON_ID, function(addon) {
      tmp.checkFirstrun(addon.version);
    });
  }else{
    var gExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
                            .getService(Ci.nsIExtensionManager);
    var version = gExtensionManager.getItemForID(ADDON_ID).version;
    this.checkFirstrun(version);
  }
}
WebMailNotifier.prototype.init=function() {
  try{
    Components.utils.import("resource://gre/modules/AddonManager.jsm");
  }catch (e) {}
  if (typeof AddonManager != "undefined"){
    this.baseURL="resource://wmn/";    
    AddonManager.addAddonListener(this);
  }else{
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Ci.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
                         .QueryInterface(Ci.nsIFileProtocolHandler);
    var file=__LOCATION__.parent;
    this.baseURL=fileHandler.getURLSpecFromFile(file);
  }

  var str="chrome://wmn/options";
  var str2="autoLogin";
  this.autoLogin=this.getPassword(str,str2);
  if(this.autoLogin==null){
    this.autoLogin=1;//this.prefBranch.getBoolPref("autoLoginDefault")?1:0;
    this.setPassword(str,str2,this.autoLogin);
  }else this.autoLogin=parseInt(this.autoLogin);

  this.bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
           .getService(Ci.nsIStringBundleService)
           .createBundle("chrome://wmn/locale/overlay.properties"),

  this.userScripts={};
  this.scripts={};
  this.initScripts(true);
  this.initScripts(false);
  this.initHandlers(true);
  if(this.prefBranch.getBoolPref("startupOpenWMN"))this.openWMN();

  for each(var l in this.listeners){
    l.onStateChange(this.MSG_INIT,this.prefBranch.getBoolPref("enabled")?1:0,null);
    for each(var o in this.webMailHandlers){
      if(o.enabled){
        l.onStateChange(o.ind,o.calcCount(),o.mailData);
      }
    }
  }
}
WebMailNotifier.prototype.checkFirstrun=function(current){
  var ver = -1;
  try{
    ver = this.prefBranch.getCharPref("version");
  }catch(e){
  }finally{
    if(ver==-1){
      for each(var l in this.listeners){
        l.onStateChange(this.MSG_SHOWTBBTN,0,null);
      }
    }
    if (ver!=current){
      this.prefBranch.setCharPref("version",current);
      this.postInstall();//for fx 2
      var timer = Components.classes["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
      var tmp={
        notify:function(aTimer){
          var hdl=new WebMailHandler();
          hdl.process=function(aHttpChannel, aData) {
            if(aData=="alive"){
              WebMailNotifier.prototype.openInTab("http://webmailnotifier.mozdev.org/updated.html?ver="+current);
            }else{
              WebMailNotifier.prototype.openInTab("http://wmn.cz.cc/updated.php?ver="+current);
            }
          }
          hdl.getHtml("http://webmailnotifier.mozdev.org/alive.php");
        }
      };
      timer.initWithCallback(tmp,1500,Ci.nsITimer.TYPE_ONE_SHOT); //Firefox 2 fix - or else tab will get closed
    }
  }
}
WebMailNotifier.prototype.initScripts = function(isDefault) {
  var ios = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Ci.nsIIOService);
  var fileHandler = ios.getProtocolHandler("file")
                       .QueryInterface(Ci.nsIFileProtocolHandler);
  var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                  .getService(Ci.mozIJSSubScriptLoader);
  if(isDefault){
    var info = Components.classes["@mozilla.org/xre/app-info;1"]
             .getService(Ci.nsIXULAppInfo);
    if(info.ID=="{3550f703-e582-4d05-9a08-453d09bdfdc6}"){
      this.multiSession=false;
      this.prefBranch.setBoolPref("multiSession",false);
      loader.loadSubScript(this.baseURL+"ui-tb.js");
    }else{
      loader.loadSubScript(this.baseURL+"ui.js");
    }
    this.dir0=this.baseURL+"scripts/";
    delete this.baseURL;
    try{
      loader.loadSubScript(this.dir0+"order.dat",this);
      loader.loadSubScript(this.dir0+"lib-socket.js");
      loader.loadSubScript(this.dir0+"lib-webupdate.js");
      loader.loadSubScript(this.dir0+"lib-rsa.js");
    }catch(e){dout(e);}
    for(var i in this.order){
      this.scripts[i]=null;
    }
  }else{
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Ci.nsIProperties)
                     .get("ProfD", Ci.nsIFile);
    file.append("wmn");
    this.dir1=fileHandler.getURLSpecFromFile(file);
    if(this.dir1.charAt(this.dir1.length-1)!="/"){//folder not exist
      this.dir1+="/";
    }
    if(file.exists()){
      var entries = file.directoryEntries;
      while(entries.hasMoreElements())
      {
        var t = entries.getNext();
        t.QueryInterface(Ci.nsIFile);
        var s=t.leafName;
        var fnd;
        if(fnd=s.match(/((lib-)?.+).js$/)){
          if(fnd[2]){//load library
            try{
              loader.loadSubScript((isDefault?this.dir0:this.dir1)+fnd[0]);
            }catch(e){
  dout(fnd[0]+" "+e);
            }
          }else{
            this.scripts[fnd[1]]=null;
          }
          if(!isDefault)this.userScripts[fnd[1]]=true;
        }
      }
    }
  }
}
WebMailNotifier.prototype.initHandler = function(obj,id,user,pass) {
  this.webMailHandlers.push(obj);
  obj.id=id;
  obj.user=user;
  obj.password=pass;
  obj.inited=false;
  if(this.order[id])obj.key=this.order[id].toString();
}
WebMailNotifier.prototype.initHandlers = function(onInit) {
//dout("***initHandlers "+aCheckNow);
  this.stopAll();
  this.webMailHandlers=[];
  var str="chrome://wmn/accounts/";
  if(this.loginManager){
    for(var id in this.scripts){
      var logins = this.loginManager.findLogins({},str+id,formSubmitURL, null);
      for each (var o in logins) {
        var obj=new WebMailHandler(this);
        this.initHandler(obj,id,o.username,o.password);
      }
    }
  }else{
    var e = this.passwordManager.enumerator;
    while (e.hasMoreElements()) {
      try {
        var pass = e.getNext().QueryInterface(Ci.nsIPassword);
        if (pass.host.indexOf(str)==0) {
          var id=pass.host.substring(str.length);
          var obj=new WebMailHandler(this);
          this.initHandler(obj,id,pass.user,pass.password);
        }
      }catch(ex) {}
    }
  }
  this.webMailHandlers.sort(this.sortFunc);
  this.buildTable(onInit);
}
WebMailNotifier.prototype.sortFunc = function(a,b) {
  var keyA=a.key?a.key:a.id;
  var keyB=b.key?b.key:b.id;
  if(keyA<keyB)return -1;
  else if(keyA==keyB){
    if(a.user<b.user)return -1;
    else if(a.user==b.user)return 0;
    else return 1;
  }else return 1;
}
WebMailNotifier.prototype.loadScript = function(id,isTemp) {
  var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                  .getService(Ci.mozIJSSubScriptLoader);
  if(!this.scripts[id]){
//dout("loadScript "+(this.userScripts[id]?this.dir1:this.dir0)+id+".js");
    var scr={};
    loader.loadSubScript((this.userScripts[id]?this.dir1:this.dir0)+id+".js",scr);
//dout("loaded "+id+" "+isTemp);
    if(!isTemp)this.scripts[id]=scr;
//if(!isTemp)dout("load ["+id+"] "+scr.name);
    return scr;
  }else return this.scripts[id];
}
WebMailNotifier.prototype.buildTable = function(onInit) {
  this.stopAll();
//dout("***buildTable "+aCheckNow);
  var i;
  for(i=0;i<this.webMailHandlers.length;++i){
    var obj=this.webMailHandlers[i];
    obj.enabled=true;
    try{
      obj.enabled=this.prefBranch.getBoolPref("accounts."+obj.id+"."+obj.user+".enabled");
    }catch(e){}
    try{
      obj.order=this.prefBranch.getIntPref("accounts."+obj.id+"."+obj.user+".order");
    }catch(e){}
    if(!obj.enabled||obj.inited)continue;
//dout("init "+obj.id+" "+obj.user);
    obj.cookieManager.ext="[wmn|"+obj.id+"|"+obj.user+"]";
    obj.cookieManager2.ext="[wmn|"+obj.id+"|"+obj.user+"|2]";
    obj.inboxOnly=true;
    obj.showFolders=true;
    //obj.alias=null;
    //obj.interval=null;
    try{
      var scr=this.loadScript(obj.id);
      for(var k in scr){
        obj[k]=scr[k];
      }
      if(obj.supportInboxOnly){
        obj.inboxOnly=true;
        try{
          obj.inboxOnly=this.prefBranch.getBoolPref("accounts."+obj.id+"."+obj.user+".inboxOnly");
        }catch(e){}
      }
      if(obj.supportShowFolders){
        obj.showFolders=true;
        try{
          obj.showFolders=this.prefBranch.getBoolPref("accounts."+obj.id+"."+obj.user+".showFolders");
        }catch(e){}
      }
      try{
        obj.includeSpam=this.prefBranch.getBoolPref("accounts."+obj.id+"."+obj.user+".includeSpam");
      }catch(e){}
      if(obj.needServer)obj.server=obj.user.split("|")[1];
      if(obj.init)obj.init();
      if(obj.loginData){
        var post=(obj.loginData[LOGIN_DATA]?obj.loginData[LOGIN_DATA]:"");
        if(obj.loginData[LOGIN_ID]){
          if(post)post+="&";
          var user;
          if(obj.needServer)user=obj.user.split("|")[0];
          else user=obj.user;
          post+=obj.loginData[LOGIN_ID]+"="+encodeURIComponent(user);
        }
        if(obj.loginData[LOGIN_PW]){
          if(post)post+="&";
          post+=obj.loginData[LOGIN_PW]+"="+encodeURIComponent(obj.password);
        }
        obj.loginData.splice(1,obj.loginData.length-1);
        obj.loginData[LOGIN_POST]=post;
      }
      obj.stage=obj.initStage;
      try{
        obj.interval=this.prefBranch.getIntPref("accounts."+obj.id+"."+obj.user+".interval");
      }catch(e){}
      try{
        obj.alias=this.prefBranch.getComplexValue("accounts."+obj.id+"."+obj.user+".alias",Ci.nsISupportsString).data;
      }catch(e){}
      try{
        obj.link=this.prefBranch.getComplexValue("accounts."+obj.id+"."+obj.user+".link",Ci.nsISupportsString).data;
      }catch(e){}
      try{
        obj.autoOpen=this.prefBranch.getBoolPref("accounts."+obj.id+"."+obj.user+".autoOpen");
      }catch(e){}
      try{
        obj.icon=this.prefBranch.getComplexValue("accounts."+obj.id+"."+obj.user+".icon",Ci.nsISupportsString).data;
      }catch(e){}      
      try{
        obj.sound=this.prefBranch.getComplexValue("accounts."+obj.id+"."+obj.user+".sound",Ci.nsISupportsString).data;
      }catch(e){}            
      obj.inited=true;
    }catch(e){
dout(obj.id+" "+e);
      this.webMailHandlers.splice(i,1);//not loaded
      --i;
    }
  }
  var ar0=[];
  var ar1=[];
  for each(var o in this.webMailHandlers){
    if(o.order!=null){
      var ind=ar1.length;
      for(var i=0;i<ar1.length;i++){
        if(o.order<ar1[i].order){
          ind=i;
          break;
        }
      }
      ar1.splice(ind,0,o);
    }
    else ar0.push(o);
  }
  for each(var o in ar1)ar0.splice(o.order,0,o);
  this.webMailHandlers=ar0;

  for(i=0;i<this.webMailHandlers.length;++i){
    this.webMailHandlers[i].ind=i;
  }

  this.hdlTable={};
  var defaults={};
  for(var i in this.scripts){
    this.hdlTable[i]=[];
    try{
      defaults[i]=this.prefBranch.getCharPref("accounts."+i+".default");
    }catch(e){}
  }
  this.maxLen=0;
  this.groups=[];
  this.groups0=[];
  for each(var o in this.webMailHandlers){
    if(o.enabled){
      if(o.user==defaults[o.id])this.hdlTable[o.id].splice(0,0,o);//default account inserted into index 0
      else this.hdlTable[o.id].push(o);
      if(this.hdlTable[o.id].length>this.maxLen)this.maxLen=this.hdlTable[o.id].length;
      o.name=this.getHostName(o.id);
      var gid=o.id;
      var group=this.groups0;
      if(o.alias){
        var fnd=o.alias.match(/(.+?)\/(.*)/);
        if(fnd){
          gid=fnd[1];
          group=this.groups;
        }
      }
      if(group[gid])++group[gid];
      else group[gid]=1;
    }
  }
  for(var i in this.hdlTable){
    var o=this.hdlTable[i];
    if(o.length>0){
      var prefStr="accounts."+i+".default";
      var t=null;
      try{
        t=this.prefBranch.getCharPref(prefStr);
      }catch(e){}
      if(t==null)this.prefBranch.setCharPref(prefStr,o[0].user);
      //set account name
      for each(var obj in o){
        if(obj.alias){
          var fnd=obj.alias.match(/(.+?)\/(.*)/);
          if(fnd){
            obj.accName=fnd[2]?(fnd[1]+"/"+fnd[2]):(obj.name+" ["+obj.user+"]");
          }else obj.accName=obj.alias;
          continue;
        }
        if(o.length>1){
          obj.accName=obj.name+" ["+obj.user+"]";
        }else obj.accName=obj.name;
      }
    }
  }

  if(!onInit)this.setState(this.MSG_RESET,0);
  if(this.prefBranch.getBoolPref("checkOnStartup")){
    var delay=Math.max(1,this.prefBranch.getIntPref("startupDelay"));
    var timer = Components.classes["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    var self=this;
    var tmp={
      notify:function(aTimer){
        self.checkMails();
      }
    };
    timer.initWithCallback(tmp,delay,Ci.nsITimer.TYPE_ONE_SHOT);
  }
}
WebMailNotifier.prototype.isDefault = function(id,user) {
  return this.hdlTable[id].length==1||(this.hdlTable[id].length>1&&this.hdlTable[id][0].user==user);
}

WebMailNotifier.prototype.isOpenable = function(id,user) {
  if(this.hdlTable[id].length==1)return true;
  else if(this.hdlTable[id].length==0)return false;
  if(this.multiSession)return true;
  var ar=this.hdlTable[id];
  for(var i=0;i<ar.length;i++){
    if(ar[i].user==user)break;
    if(ar[i].calcCount()>0)return false;
  }
  return true;
}
WebMailNotifier.prototype.addScript = function(id) {
  var rs=this.scripts[id];
  if(id.indexOf("lib-")==0){
    try{
      var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                  .getService(Ci.mozIJSSubScriptLoader);
      loader.loadSubScript(this.dir1+id+".js");
    }catch(e){
dout("load library "+e);
    }
  }else{
    this.scripts[id]=null;
  }
  this.userScripts[id]=true;
  return rs?true:false; //'return rs' returns wrong value
}

WebMailNotifier.prototype.getHostIDs = function(count) {
  var ar=[];
  for(var i in this.scripts){
    var o={id:i};
    if(this.order[i])o.key=this.order[i].toString();
    ar.push(o);
  }
  ar.sort(this.sortFunc);
  var ar2=[];
  for each(var o in ar)ar2.push(o.id);
  count.value=ar2.length;
  return ar2;
}
WebMailNotifier.prototype.getScriptList = function(count) {
  var ar=[];
  for(var i in this.userScripts){
    ar.push(i);
  }
  count.value=ar.length;
  return ar;
}
WebMailNotifier.prototype.isInstalled = function(id) {
  for(var i in this.userScripts){
    if(i==id)return true;
  }
  return false;
}
WebMailNotifier.prototype.getAccountsInHost = function(id) {
  return this.groups0[id]?this.groups0[id]:0;
}
WebMailNotifier.prototype.getAccountsInGroup = function(id) {
  return this.groups[id]?this.groups[id]:0;
}
WebMailNotifier.prototype.getString = function(str) {
  try{
    return this.bundle.GetStringFromName(str);
  }catch(e){}
  return str;
}
WebMailNotifier.prototype.getHostName = function(id) {
  try{
    return this.bundle.GetStringFromName(id);
  }catch(e){}
  try{
    var obj=this.loadScript(id,true);
    return obj.name?obj.name:id;
  }catch(e){
dout("getHostName:"+id+" "+e);
    return id;
  };
}
WebMailNotifier.prototype.getScriptVal = function(id,name) {
  try{
    var obj=this.loadScript(id,true);
    if(typeof obj[name]=="undefined")return null;
    return obj[name];
  }catch(e){
    return null;
  };
}
WebMailNotifier.prototype.getAccountsNumber = function() {
  return this.webMailHandlers.length;
}
WebMailNotifier.prototype.getAccountInfo = function(aIndex) {
  var obj=this.webMailHandlers[aIndex];
  var o={};
  o.id=obj.id;
  o.user=obj.user;
  o.password=obj.password;
  o.ind=obj.ind;
  if(obj.alias)o.alias=obj.alias;
  o.accName=obj.accName;
  return o;
}
WebMailNotifier.prototype.isAccountEnabled = function(aIndex) {
  var o=this.webMailHandlers[aIndex];
  if(o)return o.enabled;
  else return false;
}
WebMailNotifier.prototype.getIntValue = function(name) {
  return this[name];
}
WebMailNotifier.prototype.setIntValue = function(name,val) {
  this[name]=val;
}

WebMailNotifier.prototype.addListener = function(aListener) {
  if(aListener)this.listeners.push(aListener);
  if(!this.initCalled){
    this.initCalled=true;
    this.init2();
  }
  if(aListener){
    aListener.onStateChange(this.MSG_INIT,this.prefBranch.getBoolPref("enabled")?1:0,null);
    for each(var o in this.webMailHandlers){
      if(o.enabled){
        o.mailData.last=o.lastCheck;
        aListener.onStateChange(o.ind,o.calcCount(),o.mailData);
      }
    }
  }
}
WebMailNotifier.prototype.removeListener = function(aListener){
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]==aListener){
        this.listeners.splice(i,1);
        break;
    }
  }
}
WebMailNotifier.prototype.checkMail = function(aIndex) {
  var o=this.webMailHandlers[aIndex];
  o.lastCheck=new Date().getTime();
  o.checkMail();
}
WebMailNotifier.prototype.stopAll = function() {
  this.workTimer.cancel();
  for each(var o in this.webMailHandlers){
    if(o.started)o.stop();
  }
}
WebMailNotifier.prototype.checkMails = function(isTimer) {
  this.workTable=[];
  var cons=Math.max(1,this.prefBranch.getIntPref("connections"));
  var maxCons=Math.max(1,this.prefBranch.getIntPref("maxConnections"));
  var now=new Date().getTime();
  var ar=[];
  for(var n=0;n<this.maxLen;n+=cons){
    for(var i in this.hdlTable){
      var o=this.hdlTable[i];
      for(var j=n;j<n+cons&&j<o.length;j++){
        var obj=o[j];
        var interval=(obj.interval!=null?obj.interval:(obj.defaultInterval!=null?obj.defaultInterval:this.timerDelay))*60000;
        var check=false;
        if(isTimer){
          if(interval>0&&(!obj.lastCheck||obj.lastCheck+interval-500<=now))check=true;
        }else if(interval>=0)check=true;
        if(check){
          ar.push(obj);
          obj.lastCheck=now;
          obj.started=true;
        }
        if(ar.length>=maxCons){
          this.workTable.push(ar);
          ar=[];
        }
      }
    }
  }
  if(ar.length>0)this.workTable.push(ar);
  if(this.workTable.length==0)return;

  if(this.workTable.length>1){
    this.workTimer.cancel();
    this.workTimer.initWithCallback(this.worker,
                                this.connectionDelay,
                                Ci.nsITimer.TYPE_ONE_SHOT);
  }

  this.workDone=1;
  for each(var o in this.workTable[0]){
    o.checkMail();
  }
}
WebMailNotifier.prototype.timerWork = function(aTimer) {
  for each(var o in this.workTable[this.workDone]){
    o.checkMail();
  }
  ++this.workDone;
  if(this.workDone<this.workTable.length){
    this.workTimer.initWithCallback(this.worker,
                                this.connectionDelay,
                                Ci.nsITimer.TYPE_ONE_SHOT);
  }
}
WebMailNotifier.prototype.getHandler = function(aID,aUser) {
  for each(var i in this.webMailHandlers){
    if(i.id==aID&&i.user==aUser)return i;
  }
  return null;
}
WebMailNotifier.prototype.getMailURL = function(aID,aUser,aFolder) {
  var obj=this.getHandler(aID,aUser);
  if(obj){
    if(this.autoLogin&&!obj.noCookie){
      if(this.multiSession&&(!this.isDefault(aID,aUser)||!this.prefBranch.getBoolPref("autoLoginDefaultAccount"))){
        obj.cookieManager.copyTo(obj.cookieManager2);
      }else obj.cookieManager.setCookieToBrowser();
    }
    if(this.resetCounter&&!obj.noCounterReset){
      var url=obj.getMailURL(aFolder);
      var pref="accounts."+aID+"."+aUser+".count";
      try{
        this.prefBranch.getIntPref(pref);//set only if this exists
        this.prefBranch.setIntPref(pref,obj.mailCount>0?obj.mailCount:0);
      }catch(e){}
      obj.mailData.desc=obj.getDesc();
      this.setState(this.MSG_MAILDATA,obj.ind);
      return url;
    }else return obj.getMailURL(aFolder);
  }else return null;
}
WebMailNotifier.prototype.getIconURL = function(aID,aUser) {
  var obj=this.getHandler(aID,aUser);
  if(obj){
    if(obj.icon)return obj.icon;
    var url=obj.getIconURL();    
    if(this.faviconService){
      var page=obj.getIconPage();
      if(page){
        try{
          var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                            .getService( Ci.nsIIOService);
          var aURI=ioService.newURI(page,null,null);      
          var icon=this.faviconService.getFaviconImageForPage(aURI).spec;
          if(icon.indexOf("chrome://")==0){
            return url;
          }
          return icon;
        }catch(e){}
      }
    }
    return url;
  }
  else return null;
}
WebMailNotifier.prototype.setData = function(aIndex) {
  this.setState(this.MSG_MAILDATA,aIndex);
}
WebMailNotifier.prototype.setState = function(aCmd,aIndex) {
  var aData=null;
  var aCount=0;
  var openList=[];
  if(aCmd==this.MSG_MAILDATA){
    var o=this.webMailHandlers[aIndex];
    o.started=false;
    o.mailData.last=o.lastCheck;
    aData=o.mailData;
    aCount=o.calcCount();
    var finished=true;
    for each(o in this.webMailHandlers){
      if(o.started){
        finished=false;
        break;
      }
    }
    if(finished){
      var total=0;
      for each(o in this.webMailHandlers){
        if(o.enabled&&o.mailCount>0){
          total+=o.calcCount();
        }
      }
      if(total>this.countTotal){
        this.notification();
        for each(var o in this.webMailHandlers){
          if(o.enabled&&o.autoOpen&&o.mailCount>0&&o.calcCount()>0&&this.isOpenable(o.id,o.user)){
            openList.push(o);
          }
        }
      }else if(this.countTotal>0&&total==0){
          var win=this.findWindow("alert:alert","wmn_alert");
          if(win)win.close();
      }
      this.countTotal=total;
//dout("all done");
    }
  }else{//special command
    aCount=aIndex;
    aIndex=aCmd;
  }
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]){
      this.listeners[i].onStateChange(aIndex,aCount,aData);
    }
  }
  for each(var o in openList)this.openMail(o.ind);
}
WebMailNotifier.prototype.notification = function() {
  if(this.prefBranch.getBoolPref("showNotification")){
    if(this.listeners.length>0){
      this.showAlertNotification();
    }
  }
  if(this.prefBranch.getBoolPref("mailSound")){
    if (!this.mSound)this.mSound = Components.classes["@mozilla.org/sound;1"].createInstance(Ci.nsISound);
    var soundLocation = this.prefBranch.getBoolPref("customSound")?
              this.prefBranch.getCharPref("soundUrl") : "_moz_mailbeep";    
    for each(var o in this.webMailHandlers){
      if(o.enabled){
        if(o.calcCount()>0&&o.sound){
          soundLocation=o.sound;
          break;
        }
      }
    }
    if(soundLocation.indexOf("file://") == -1){
      var os = Components.classes["@mozilla.org/xre/app-info;1"]
                  .getService(Ci.nsIXULRuntime).OS;
      if(os=="Darwin")soundLocation="";
      this.mSound.playSystemSound(soundLocation);
    }else{
      var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
      try{
        this.mSound.play(ioService.newURI(soundLocation, null, null));
      }catch(e){}
    }
  }
}
WebMailNotifier.prototype.openMails = function(openAll){
  var list=[];
  for each(var o in this.webMailHandlers){
    if(o.enabled){
      if((o.mailCount>0&&o.calcCount()>0&&this.isOpenable(o.id,o.user))
        ||(openAll&&(this.multiSession||this.isDefault(o.id,o.user)))){
        list.push(o);
      }
    }
  }
  for each(var o in list)this.openMail(o.ind);
}
WebMailNotifier.prototype.openMail = function(ind,folder,reload){
  var obj=this.webMailHandlers[ind];
  var id=obj.id;
  var user=obj.user;
  var url=this.getMailURL(id,user,folder);//call this for mailCount set
  if(!url)return;
  if(url.indexOf("file://")==0){
    try{
      url=url.split(" ");    
      var ios = Components.classes["@mozilla.org/network/io-service;1"]
                          .getService(Ci.nsIIOService);
      var fileHandler = ios.getProtocolHandler("file")
                           .QueryInterface(Ci.nsIFileProtocolHandler);
      var file=fileHandler.getFileFromURLSpec(url[0]);
      var process = Components.classes["@mozilla.org/process/util;1"]
                              .createInstance(Components.interfaces.nsIProcess);
      process.init(file); 
      var args = url.splice(1);
      for(var i=0;i<args.length;i++){
        args[i]=args[i].replace(/^["']/,"").replace(/["']$/,"");
      }
      process.run(false, args, args.length);
    }catch(e){
dout(e);    
    }    
    return;
  }
  var mailDomain=obj.mailDomain;
  if(reload)mailDomain=null;
  var reuse=this.prefBranch.getBoolPref("reuseTab");
  var name=this.multiSession&&!this.isDefault(id,user)?ind:id;
  var sid=!this.multiSession||(!this.isDefault(id,user)||!this.prefBranch.getBoolPref("autoLoginDefaultAccount"))?ind:null;
  if(this.prefBranch.getBoolPref("openInTab")){
    this.openTab(url,name,reuse,mailDomain,sid);
  }else this.openWindow(url,name,reuse,mailDomain,sid);
}
WebMailNotifier.prototype.canReuse = function(cur,mailDomain) {
    if(cur.spec=="about:blank"||!mailDomain)return true;
    var host=null;
    try{
      host=cur.spec.substring(cur.scheme.length+3);      
    }catch(e){}
    if(!host)return false;
    try{
      var reg=new RegExp(mailDomain);
      if(!host.match(reg))return false;
    }catch(e){}
    return true;
}

WebMailNotifier.prototype.findWindow = function(type,name){
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var enm = wm.getEnumerator(type);
  while(enm.hasMoreElements()){
    var win = enm.getNext();
    if(win.name==name)return win;
  }
  return null;
}
WebMailNotifier.prototype.openDialog = function(url, name, features) {
  return this.getWindow().openDialog(url,name,features,arguments[3],arguments[4],arguments[5]);
}
WebMailNotifier.prototype.openCaptchaDialog = function(host,user,arg) {
  var params = {inn:{host:this.getHostName(host),user:user,secimage:arg}, out:null};
  var features = "chrome,titlebar,toolbar,centerscreen,modal";
  this.openDialog("chrome://wmn/content/captcha.xul","wmn_"+host+":"+user,features,params).focus();
  if (params.out)return params.out.secword;
  else return null;
}
WebMailNotifier.prototype.showAlertNotification = function(){
  this.openDialog("chrome://wmn/content/alert.xul","wmn_alert",
      "chrome,dialog=yes,titlebar=no,popup=yes",
      this.webMailHandlers,this);
}
WebMailNotifier.prototype.openWMN = function(){
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow("WMN:Window");
  if(win){
    win.focus();
    return;
  }
  this.openDialog("chrome://wmn/content/wmn-window.xul","wmn_win",
      "chrome,toolbar=yes,dialog=no,resizable").focus();
}
WebMailNotifier.prototype.openOptions = function() {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Ci.nsIPrefService);
  var branch = prefService.getBranch("browser.preferences.");
  branch.QueryInterface(Ci.nsIPrefBranch2);
  var instantApply= branch.getBoolPref("instantApply");
  var features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");
  this.openDialog("chrome://wmn/content/options.xul","wmn-pref",features).focus();
}
/***********************************************************
timer
***********************************************************/
WebMailNotifier.prototype.notify = function(aTimer) {
  if(this.prefBranch.getBoolPref("enabled"))this.checkMails(true);
}
/***********************************************************
preference observer
***********************************************************/
WebMailNotifier.prototype.observe=function(aSubject, aTopic, aData)
{
  switch(aTopic){
  case "nsPref:changed":
    switch (aData) {
    case "updateInterval":
      this.timerDelay=this.prefBranch.getIntPref(aData);
      break;
    case "resetCounter":
      this.resetCounter=this.prefBranch.getBoolPref("resetCounter");
      for each(var o in this.webMailHandlers){
        if(o.enabled){
          /*if(this.resetCounter){
            this.prefBranch.setIntPref("accounts."+o.id+"."+o.user+".count",0);
          }else{*/
          if(!this.resetCounter){
            try{
              this.prefBranch.clearUserPref("accounts."+o.id+"."+o.user+".count");
              o.mailData.desc=o.getDesc();
            }catch(e){
dout(e);
            }
          }
          this.setState(this.MSG_MAILDATA,o.ind);
        }
      }
      break;
    case "showStatusbarIcon":
      this.setState(this.MSG_SHOWICON,this.prefBranch.getBoolPref("showStatusbarIcon"));
      break;
    case "enabled":
      this.setState(this.MSG_ENABLED,this.prefBranch.getBoolPref("enabled"));
      break;
    case "multiSession":
      this.multiSession=this.prefBranch.getBoolPref("multiSession");
      break;
    case "debug":
      enableDebug=this.prefBranch.getIntPref("debug");
      break;
    }
    break;
  case "http-on-modify-request":
    aSubject.QueryInterface(Ci.nsIHttpChannel);
    for each(var i in this.webMailHandlers){
      if(i.channel==aSubject){
        i.onRequest(aSubject);
        break;
      }
    }
    if(this.multiSession){
      var domWin=this.getWinFromChannel(aSubject);
//dout(aSubject.URI.spec+" "+domWin);
      if(domWin){
        var tab=this.getTabFromWin(domWin);
        if(tab){
          var sid=tab.getAttribute("wmnsid");
          if(sid){
            this.webMailHandlers[sid].onRequest(aSubject,true);
          }
        }
      }
    }
    break;
  case "http-on-examine-response":
    aSubject.QueryInterface(Ci.nsIHttpChannel);
    for each(var i in this.webMailHandlers){
      if(i.channel==aSubject){
        i.onResponse(aSubject);
        break;
      }
    }
    if(this.multiSession){
      var domWin=this.getWinFromChannel(aSubject);
      if(domWin){
        var tab=this.getTabFromWin(domWin);
        if(tab){
          var sid=tab.getAttribute("wmnsid");
          if(sid){
            this.webMailHandlers[sid].onResponse(aSubject,true);
          }
        }
      }
    }
    break;
  case "em-action-requested":
    aSubject.QueryInterface(Ci.nsIUpdateItem);
    if(aSubject.id == ADDON_ID) {
      if (aData == "item-uninstalled") {
        this.uninstall = true;
      } else if (aData == "item-cancel-action") {
        this.uninstall = false;
      }
    }
    break;
  case "quit-application-granted":
    if(this.uninstall){
      var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                              .getService(Components.interfaces.nsIPromptService);
      var result = prompts.confirm(null, this.appName,this.getString("clearSettings"));
      if(!result)return;
      var str="chrome://wmn/";
      if(this.loginManager){
        var logins = this.loginManager.getAllLogins({});
        for each (var o in logins) {
          if(o.hostname.indexOf(str)==0)this.loginManager.removeLogin(o);
        }
      }else{
        var e = this.passwordManager.enumerator;
        while (e.hasMoreElements()) {
          var o = e.getNext().QueryInterface(Ci.nsIPassword);
          if (o.host.indexOf(str)==0)this.passwordManager.removeUser(o.host,o.user);
        }
      }

      var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Ci.nsIPrefService);
      var prefBranch = prefService.getBranch("");
      try{ prefBranch.deleteBranch("extensions.wmn."); }catch(e){}
      prefService.savePrefFile(null);

      var ios = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Ci.nsIIOService);
      var fileHandler = ios.getProtocolHandler("file")
                          .QueryInterface(Ci.nsIFileProtocolHandler);
      var file=fileHandler.getFileFromURLSpec(this.dir1);
      try{file.remove(true);}catch(e){}
    }
    break;
  }
}
WebMailNotifier.prototype.initCookieHandler=function(doc,tab,ind){
  try{
    var docObj=doc.wrappedJSObject;
    if(!doc.com)doc.com={};
    if(!doc.com.tobwithu)doc.com.tobwithu={};
    if(!doc.com.tobwithu.wmn){
      doc.com.tobwithu.wmn={};
      doc.com.tobwithu.wmn.cookieGetter = docObj.__lookupGetter__("cookie");
      doc.com.tobwithu.wmn.cookieSetter = docObj.__lookupSetter__("cookie");

      var cm0=this.webMailHandlers[ind].cookieManager2;//for ff2
      try{
        var src=
          "doc.__defineGetter__('cookie',\
            function(){\
              try{\
              if(tab.hasAttribute('wmnsid')){\
                var sid=tab.getAttribute('wmnsid');\
                var cm=hdls[sid].cookieManager2;\
                return cm.getCookie(doc.location.href);\
              }\
              }catch(e){}\
              return cm0.getCookie(doc.location.href);\
            }\
          );\
          doc.__defineSetter__('cookie',\
            function(val){\
              try{\
              if(tab.hasAttribute('wmnsid')){\
                var sid=tab.getAttribute('wmnsid');\
                var cm=hdls[sid].cookieManager2;\
                cm.addCookies(doc.location.href,val);\
                return;\
              }\
              }catch(e){}\
              cm0.addCookies(doc.location.href,val);\
            }\
          );";
        var sandbox = new Components.utils.Sandbox(doc.defaultView);
        sandbox.doc=docObj;
        sandbox.hdls=this.webMailHandlers;
        sandbox.tab=tab;
        sandbox.cm0=cm0;
        Components.utils.evalInSandbox(src,sandbox);

        try{
          doc.com.tobwithu.wmn.open=docObj.defaultView.open;
          var self=this;
          docObj.defaultView.open=function(url){
            var wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
                                .getService(Components.interfaces.nsIWindowMediator);
            var ar=Array.prototype.slice.call(arguments, 0);
            ar[0]="about:blank";
            var win0=doc.com.tobwithu.wmn.open.apply(docObj.defaultView,ar);
            win = wm.getMostRecentWindow("navigator:browser");
            var tabbrowser=win.getBrowser();
            var tab=tabbrowser.selectedTab;
            var browser=tabbrowser.getBrowserForTab(tab);
            tab.setAttribute("wmnsid",ind);
            try{browser.addProgressListener(self);}catch(e){}
            var a=docObj.createElement("a");
            a.href=url;
            browser.loadURIWithFlags(a.href);
            return win0;
          }
        }catch(e0){
dout(e0);
        }
      }catch(e){}
    }
  }catch(e){
dout(e);
  }
}

WebMailNotifier.prototype.onStateChange=function(webProgress, request, state, status) {
  if(state&(Ci.nsIWebProgressListener.STATE_STOP|Ci.nsIWebProgressListener.STATE_IS_DOCUMENT)){
//dout(request.name+" @@@@@state "+webProgress+" "+request+" "+state.toString(16)+" "+status);
    var domWin=webProgress.DOMWindow;
    if(domWin){
      var tab=this.getTabFromWin(domWin);
      if(tab){
        var sid=tab.getAttribute("wmnsid");
        if(sid){
          this.initCookieHandler(domWin.document,tab,sid);
        }
      }
    }
  }
}
WebMailNotifier.prototype.onLocationChange=function(webProgress, request, location) {
}
WebMailNotifier.prototype.onProgressChange=function(webProgress, request, curSelfProgress) {
}
WebMailNotifier.prototype.onSecurityChange=function(webProgress, request, state) {
}
WebMailNotifier.prototype.onStatusChange=function(webProgress, request, status, message) {
}

WebMailNotifier.prototype.onUninstalling=function(addon,restart){
  if(addon.id==ADDON_ID){
    this.uninstall=true;
  }
}
WebMailNotifier.prototype.onOperationCancelled=function(addon){
  if(addon.id==ADDON_ID){
    if(this.uninstall)this.uninstall=false;
  }
}
WebMailNotifier.prototype.postInstall=function(){
  var s1="accounts.wmn_forums.default.enabled";
  var s2="chrome://wmn/accounts/wmn_forums";
  var val=null;
  try{ val=this.prefBranch.getBoolPref(s1); }catch(e){};
  if(this.getPassword(s2,"default")==null){
    this.setPassword(s2,"default","1");
    this.prefBranch.setBoolPref(s1,false);
  }
  //import previous version
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Ci.nsIPrefService);
  var pb = prefService.getBranch("");
  var str="chrome://wm-notifier/webmail";
  var str2="chrome://wmn/accounts/";
  var hosts=["daum","naver",null,"nate","gmail","yahoo","hotmail"];
  if(this.loginManager){
    for(var i=0;i<hosts.length;i++){
      var logins = this.loginManager.findLogins({},str+i,formSubmitURL, null);
      for each (var o in logins) {
        var host=hosts[i];
        if(host){
          this.setPassword(str2+host,o.username,o.password);
          this.removePassword(str+i,o.username);
          try{
            var enabled=pb.getBoolPref("wm-notifier.webmail"+i+"."+o.username+".enabled");
            pb.setBoolPref("extensions.wmn.accounts."+host+"."+o.username+".enabled",enabled);
          }catch(e){}
        }
      }
    }
  }else{
    var e = this.passwordManager.enumerator;
    while (e.hasMoreElements()) {
      try {
        var pass = e.getNext().QueryInterface(Ci.nsIPassword);
        if (pass.host.indexOf(str)==0) {
          var i=pass.host.substring(str.length);
          var host=hosts[i];
          if(host){
            this.setPassword(str2+host,pass.user,pass.password);
            this.removePassword(str+i,pass.user);
            try{
              var enabled=pb.getBoolPref("wm-notifier.webmail"+i+"."+pass.user+".enabled");
              pb.setBoolPref("extensions.wmn.accounts."+host+"."+pass.user+".enabled",enabled);
            }catch(e){}
          }
        }
      }catch(ex) {}
    }
  }
  //delete userscript
  this.deleteFile("wmn/aol.js");
  this.deleteFile("wmn/paran.js");
  this.deleteFile("wmn/wmn_forums.js");
  //if(restart)this.initHandlers();
}
WebMailNotifier.prototype.getWinFromChannel=function(aChannel){
  try{
    return aChannel.notificationCallbacks.getInterface(Ci.nsIDOMWindow);
  }catch(e){}
  try{
    return aChannel.loadGroup.notificationCallbacks.getInterface(Ci.nsIDOMWindow);
  }catch(e){}
  return null;
}
WebMailNotifier.prototype.getTabFromWin=function(domWin){
  if(!domWin)return null;
  try{
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
             .getService(Ci.nsIWindowMediator);
    var enm = wm.getEnumerator("navigator:browser");
    while(enm.hasMoreElements()){
      var tabbrowser=enm.getNext().getBrowser();
      var ind=tabbrowser.getBrowserIndexForDocument(domWin.top.document);
      if(ind<0)continue;
      var tab = tabbrowser.mTabContainer.childNodes[ind];
      if(tab)return tab;
    }
  }catch (e){
dout(e);
  }
  return null;
}

/***********************************************************
password manager
***********************************************************/
WebMailNotifier.prototype.getPassword=function(host,user){
  if(this.loginManager){
    var logins = this.loginManager.findLogins({},host,formSubmitURL, null);
    for each (var o in logins) {
      if (o.username==user) {
        return o.password;
      }
    }
  }else{
    var e = this.passwordManager.enumerator;
    while (e.hasMoreElements()) {
      try {
        var pass = e.getNext().QueryInterface(Ci.nsIPassword);
        if (pass.host == host&&pass.user == user) {
           return pass.password;
        }
      }catch(ex) {}
    }
  }
  return null;
}
WebMailNotifier.prototype.setPassword=function(host,user,passwd){
  if(passwd==null)return;
  if(this.loginManager){
    var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
                                                 Ci.nsILoginInfo,
                                                 "init");
    var loginInfo = new nsLoginInfo(host, formSubmitURL, null, user, passwd, "", "");

    var logins = this.loginManager.findLogins({},host,formSubmitURL, null);
    for each (var o in logins) {
      if (o.username==user) {
        this.loginManager.modifyLogin(o,loginInfo);
        return;
      }
    }
    this.loginManager.addLogin(loginInfo);
  }else{
    try{
      this.passwordManager.removeUser(host,user);
    }catch(e){}
    this.passwordManager.addUser(host,user,passwd);
  }
}
WebMailNotifier.prototype.removePassword=function(host,user){
  if(this.loginManager){
    var logins = this.loginManager.findLogins({},host,formSubmitURL, null);
    for each (var o in logins) {
      if (o.username==user) {
        this.loginManager.removeLogin(o);
        break;
      }
    }
  }else{
    try{
      this.passwordManager.removeUser(host,user);
    }catch(e){}
  }
}
WebMailNotifier.prototype.loadFile=function(name){
  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                         .getService(Ci.nsIProperties)
                         .get("ProfD", Ci.nsIFile);
  name=name.split("/");
  for each(var o in name)file.append(o);
  return this.loadFile0(file);
}

WebMailNotifier.prototype.loadFile0=function(file){
  var str="";
  try {
    var fis = Components.classes["@mozilla.org/network/file-input-stream;1"]
                          .createInstance(Ci.nsIFileInputStream);
    fis.init(file, 0x01, 0664, 0);
    var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
                       .createInstance(Ci.nsIConverterInputStream);
    is.init(fis, "UTF-8", 1024, Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);

    var s = {};
    while (is.readString(4096, s) != 0) {
      str+=s.value;
    }
    is.close();
    fis.close();
  }catch(e){
dout(e);
  }
  return str;
}
WebMailNotifier.prototype.saveFile=function(name,str,append){
  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Ci.nsIProperties)
                     .get("ProfD", Ci.nsIFile);
  name=name.split("/");
  for(var i in name){
    file.append(name[i]);
    if(i<name.length-1){
      if(!file.exists()){
        file.create(Ci.nsIFile.DIRECTORY_TYPE,0755);
      }
    }
  }
  return this.saveFile0(file,str,append);
}
WebMailNotifier.prototype.saveFile0=function(file,str,append){
  try{
    var fos = Components.classes["@mozilla.org/network/file-output-stream;1"]
                          .createInstance(Ci.nsIFileOutputStream);
    fos.init(file, 0x02 | 0x08 |(append?0x10:0x20), 0664, 0);
    var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
                       .createInstance(Ci.nsIConverterOutputStream);
    os.init(fos, "UTF-8", 0, 0x0000);
    os.writeString(str);
    os.close();
    fos.close();
  }catch(e){
dout(e);
  }
  return file;
}
WebMailNotifier.prototype.deleteFile=function(name){
  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Ci.nsIProperties)
                     .get("ProfD", Ci.nsIFile);
  name=name.split("/");
  for each(var o in name)file.append(o);
  if(file.exists()){
    file.remove(false);
  }
}
WebMailNotifier.prototype.removeHost=function(aHostID){
  try{
    this.prefBranch.deleteBranch("accounts."+aHostID);
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Ci.nsIPrefService);
    prefService.savePrefFile(null);
  }catch(e){
dout(e);
  }
  for each(var o in this.webMailHandlers){
    if(o.id==aHostID){
      this.removePassword("chrome://wmn/accounts/"+o.id,o.user);
    }
  }
  delete this.scripts[aHostID];
  delete this.userScripts[aHostID];
}
/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var WebMailNotifierFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new WebMailNotifier()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var WebMailNotifierModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Ci.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Ci.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Ci.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return WebMailNotifierFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
try{
  Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
  if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([WebMailNotifier]);
}catch(e){}
if(typeof NSGetFactory == "undefined"){
  function NSGetModule(aCompMgr, aFileSpec) { return WebMailNotifierModule; }
}

/***********************************************************
CookieManager definition
***********************************************************/
function endsWith(s1,s2){
  if(!s2)return false;
  var n=s1.toLowerCase().lastIndexOf(s2.toLowerCase());
  return n==-1?false:(n==s1.length-s2.length);
}
function CookieInfo(){
}
CookieInfo.prototype={
  getDomain:function(){
    if(this.domain){
      if(this.domain.charAt(0)!=".")return "."+this.domain;
      else return this.domain;
    }
    return this.uri.host;
  },
  getPath:function(){
    if(this.path)return this.path;
    var path=this.uri.path;
    var i=path.indexOf("?");
    if(i!=-1)path=path.substring(0,i);
    i=path.lastIndexOf("/");
    if(i==-1)return path;
    else return path.substring(0,i+1);
  },
  getExpiry:function(){
    if(!this.expires)return 0x7ffffffffffffdff;
    var exp=this.expires;
    exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
    return Date.parse(exp)/1000;
  },
  toString:function(){
      var s=this.name+"="+this.value;
      for(var p in this){
        if(typeof this[p] == "function")continue;
        if(p=="httponly"||p=="secure"){
          s+="; "+p;
        }else if(p!="name"&&p!="value"&&p!="uri"){
          s+="; "+p+"="+this[p];
        }
      }
    return s;
  }
}
function CookieManager(){
}
CookieManager.prototype={
  addCookies: function(aURI,aCookie){  
    if(typeof(aURI)=="string"){
      var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Ci.nsIIOService);
      aURI=ioService.newURI(aURI,null,null);
    }    
    aCookie=aCookie.split("\n");
    for each(var o in aCookie){
      var val=new CookieInfo();
      var ar=o.split(";");
      var t=ar[0].match(/(\S+?)=(.*)/);
      val["name"]=t[1];
      val["value"]=t[2];

      for(var i=1;i<ar.length;i++){
        if(ar[i]){
          var t=ar[i].replace( /^\s+/g,"");
          var f=t.match(/(\S+?)=(.*)/);
          if(f)val[f[1].toLowerCase()]=f[2];
          else{
            t=t.toLowerCase();
            if(t=="httponly"||t=="secure")val[t]=true;
            else val[t]="";
          }
        }
      }
      val.uri=aURI;
      this._insert(val);
//if(val.getPath()!=val.path)dout(val.getDomain()+" "+val.getPath()+" "+val.domain+" "+val.path);
    }
  },
/*  isExpired: function(exp){
    if(exp){
      exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
      if(Date.parse(exp)<new Date())return true;
    }
    return false;
  },*/
  _insert: function(val){  
    var cm = Components.classes["@mozilla.org/cookiemanager;1"]
              .getService(Ci.nsICookieManager2);
    try{
      if (cm.cookieExists){
//dout(val);      
//dout(val.getDomain()+this.ext+","+val.getPath()+","+val.name+" "+val.value+","+val.secure+","+val.httponly+","+!val.expires+","+val.getExpiry());
        cm.add(val.getDomain()+this.ext,val.getPath(),val.name,val.value,val.secure,val.httponly,!val.expires,val.getExpiry());
      }else{//ff2
        cm.add(val.getDomain()+this.ext,val.getPath(),val.name,val.value,val.secure,!val.expires,val.getExpiry());
      }
    }catch(e){}  
  },
  findCookie:function(domain,name){
    var cm = Components.classes["@mozilla.org/cookiemanager;1"].
                getService(Components.interfaces.nsICookieManager2);
    var enm = cm.enumerator;
    while(enm.hasMoreElements()){
      var ck = enm.getNext().QueryInterface(Components.interfaces.nsICookie2);
      if(endsWith(ck.host,domain+this.ext)&&ck.name==name){
        return ck.value;
      }
    }
    return null;
  },
  cookieToString:function(ck){
    var s=ck.name+"="+ck.value;
    if(ck.expires!=0)s+="; expires="+new Date(ck.expires*1000).toUTCString();
    if(ck.path)s+="; path="+ck.path;
    if(ck.host)s+="; domain="+ck.host.substring(0,ck.host.length-this.ext.length);
    if(ck.secure)s+="; secure";
    if(ck.httponly)s+="; httponly";
    return s;
  },  
  findCookieString:function(domain,name){
    var cm = Components.classes["@mozilla.org/cookiemanager;1"].
                getService(Components.interfaces.nsICookieManager2);
    var enm = cm.enumerator;
    while(enm.hasMoreElements()){
      var ck = enm.getNext().QueryInterface(Components.interfaces.nsICookie2);
      if(endsWith(ck.host,domain+this.ext)&&ck.name==name){
        return this.cookieToString(ck);
      }
    }
    return null;
  },    
  getCookie: function(aURI){
    if(typeof(aURI)=="string"){
      var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Ci.nsIIOService);
      aURI=ioService.newURI(aURI,null,null);
    }
    aURI=aURI.clone();
    aURI.host+=this.ext;
    /*var cs = Components.classes["@mozilla.org/cookieService;1"]
                  .getService(Components.interfaces.nsICookieService);
    return cs.getCookieStringFromHttp(aURI,null,null);*/
    var cm = Components.classes["@mozilla.org/cookiemanager;1"].
                getService(Components.interfaces.nsICookieManager2);
    var enm = cm.enumerator;
    var str="";
    while(enm.hasMoreElements()){
      var ck = enm.getNext().QueryInterface(Components.interfaces.nsICookie2);      
      if(endsWith(ck.host,this.ext)){
        var domain=ck.host;
        if(domain.charAt(0)==".")domain=domain.substring(1);
        if(!endsWith(aURI.host,domain))continue;
        if(aURI.path.indexOf(ck.path)!=0)continue;
        if(ck.isSecure&&aURI.scheme!="https")continue;
        if(ck.expires!=0&&ck.expires*1000<new Date())continue;
        var s=ck.name+"="+ck.value;
        if(str)str+="; "+s;
        else str=s;
      }
    }  
    return str;    
  },
  setCookieToBrowser: function(){
    this.copyTo();
  },
  copyTo:function(dst){
    var cm = Components.classes["@mozilla.org/cookiemanager;1"].
                getService(Components.interfaces.nsICookieManager2);
    var enm = cm.enumerator;
    if(dst)dst.clear();
    while(enm.hasMoreElements()){
      var ck = enm.getNext().QueryInterface(Components.interfaces.nsICookie2);
      if(endsWith(ck.host,this.ext)){
        try{
          var host=ck.host.substring(0,ck.host.length-this.ext.length);
          if(dst)host+=dst.ext;
          if (cm.cookieExists){
            cm.add(host,ck.path,ck.name,ck.value,ck.isSecure,ck.isHttpOnly,ck.isSession,ck.expiry);
          }else{//ff2
            cm.add(host,ck.path,ck.name,ck.value,ck.isSecure,ck.isSession,ck.expiry);
          }
        }catch(e){}
      }
    }    
  },  
  clear: function(){
    var obj = Components.classes["@mozilla.org/cookiemanager;1"].
                getService(Components.interfaces.nsICookieManager2);
    var enm = obj.enumerator;
    while(enm.hasMoreElements()){
      var ck = enm.getNext().QueryInterface(Components.interfaces.nsICookie2);
      if(endsWith(ck.host,this.ext)){
        obj.remove(ck.host,ck.name,ck.path,false);
      }
    }    
  }
}
/***********************************************************
WebMailHandler definition
***********************************************************/
function WebMailHandler(webMailNotifier){
  this.main=webMailNotifier;
  this.cookieManager=new CookieManager();
  this.cookieManager2=new CookieManager();
  this.loggedIn=false;
  this.reset();
}
WebMailHandler.prototype={
  browser:"firefox",
  charSet: null,
  data: "",
  stage: 0,
  initStage: ST_LOGIN,
  mailCount:-1,
  mailData:{},
  user: "",
  password: null,
  loginData:[],
  started:false,
  retry:1,
  asyncOnChannelRedirect: function(oldChan, newChan, flags, redirectCallback) {
    this.onChannelRedirect(oldChan, newChan, flags);
    redirectCallback.onRedirectVerifyCallback(0);
  },
  // nsIChannelEventSink
  onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
    if(this.channel==aOldChannel)this.channel = aNewChannel;
  },
  // nsIInterfaceRequestor
  getInterface: function (aIID) {
    try {
      return this.QueryInterface(aIID);
    } catch (e) {
//dout("getInterface "+aIID);
      throw Components.results.NS_NOINTERFACE;
    }
  },
  // nsIProgressEventSink
  onProgress : function (aRequest, aContext, aProgress, aProgressMax) { },
  onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },
  // nsIHttpEventSink
  onRedirect : function (aOldChannel, aNewChannel) { },
  // nsIAuthPromptProvider
  getAuthPrompt : function(aReason){ return null; },
  QueryInterface : function(aIID) {
    if (aIID.equals(Ci.nsISupports) ||
        aIID.equals(Ci.nsIInterfaceRequestor) ||
        aIID.equals(Ci.nsIChannelEventSink) ||
        aIID.equals(Ci.nsIProgressEventSink) ||
        aIID.equals(Ci.nsIHttpEventSink) ||
        aIID.equals(Ci.nsIStreamListener)||
        aIID.equals(Ci.nsIAuthPromptProvider)||
        aIID.equals(Ci.nsIPrompt)|| //not used but required in ff 2.0
        aIID.equals(Ci.nsIDocShell)|| //required in ff 3.0
        aIID.equals(Ci.nsIAuthPrompt))
      return this;
//dout("QueryInterface "+aIID);
    throw Components.results.NS_NOINTERFACE;
  },
  onStartRequest : function (aRequest, aContext) {
    this.data = "";
  },
  onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
	  var is = Components.classes["@mozilla.org/scriptableinputstream;1"]
            .createInstance(Ci.nsIScriptableInputStream);
  	is.init(aStream);
  	this.data += is.read(aLength);
  },
  onStopRequest : function (aRequest, aContext, aStatus) {
    if(aStatus==Components.results.NS_BINDING_ABORTED)return;
    var charSet=null;
    var httpChannel=aRequest.QueryInterface(Ci.nsIHttpChannel);
    if(this.charSet)charSet=this.charSet;
    else{
      var fnd=this.data.match(/<meta.+?\".+?charset=(\S+?)\s*?;?\s*?\"\s*?\x2f?>/);
      if(fnd)charSet=fnd[1];
      else{
        try{
          if(httpChannel.contentCharset)charSet=httpChannel.contentCharset;
        }catch(e){}
      }
    }
    if(charSet){
      var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
      								.createInstance(Ci.nsIScriptableUnicodeConverter);
      try{
        uniConv.charset = charSet;
        this.data=uniConv.ConvertToUnicode(this.data);
      }catch(e){}
    }
    try{
      if(!this.process(aContext.QueryInterface(Ci.nsIHttpChannel), this.data))++this.stage;
    }catch(e){
      this.onError();
dout(this.id+" "+e);
    }
  },
  stop : function(){
    if(this.channel){
      this.channel.cancel(Components.results.NS_BINDING_ABORTED);
    }
  },
  process : function (aHttpChannel, aData){
    switch(this.stage){
    case ST_LOGIN:
      this.getHtml(this.loginData[LOGIN_URL],this.loginData[LOGIN_POST]);
      return false;
    case ST_LOGIN_RES:
      this.stage=ST_DATA;
    case ST_DATA:
      if(!this.dataURLCopy){
        if(this.allDataURL&&!this.inboxOnly)this.dataURLCopy=this.allDataURL;
        else this.dataURLCopy=this.dataURL;
      }
      this.getHtml(this.dataURLCopy);
      return false;
    case ST_DATA_RES:
      var n=parseInt(this.getCount(aData));
      this.mailCount=isNaN(n)?-1:n;
      this.mailData=this.getData(aData);
      this.mailData.desc=this.getDesc();
      try{
        if(this.mailCount>=0&&aHttpChannel!=null&&aHttpChannel.responseStatus==302){
          /*var l=aHttpChannel.getResponseHeader("Location");
          if(l.indexOf("http")==0){
            this.dataURLCopy=l;
          }else{
            if(l.indexOf("/")==0){
              this.dataURLCopy=aHttpChannel.URI.scheme+"://"+aHttpChannel.URI.hostPort+l;
            }
          }*/
          this.dataURLCopy=this.channel.URI.spec;
        }
      }catch(e){
dout(e);
      }
      if(this.mailCount<0){
        if(this.retry<1){
          ++this.retry;
          this.cookieManager.clear();
          this.checkMail();
          return true;
        }else{
          if(enableDebug!=0)dlog(this.id+" "+this.user+" "+this.stage+"\t"+this.data);
          this.reset();
        }
      }else{
        this.retry=0;
        this.stage=ST_DATA;
      }
      this.main.setState(this.main.MSG_MAILDATA,this.ind);
      if(this.main.autoLogin&&!this.noCookie&&this.main.prefBranch.getBoolPref("autoLoginDefaultAccount")){
        if(this.mailCount>=0&&!this.loggedIn&&this.main.isDefault(this.id,this.user)){
          this.loggedIn=true;
          this.cookieManager.setCookieToBrowser();
        }
      }
      if(this.main.autoLogin&&!this.noCookie){
        if(this.mailCount>=0&&!this.loggedIn){
          if(this.main.multiSession&&(!this.main.isDefault(this.id,this.user)||!this.main.prefBranch.getBoolPref("autoLoginDefaultAccount"))){
            this.loggedIn=true;
            var ar=this.cookieManager.copyTo(this.cookieManager2);
          }
        }
      }      
      return true;
    }
    if(this.stage<ST_PRE&&this.checkLogin){
      return this.checkLogin(aData,aHttpChannel);
    }    
dout("[onError] "+this.id+" "+this.user+" "+this.stage);
    this.onError();
    return true;
  },
  onError : function(){
    if(enableDebug!=0)dlog(this.id+" "+this.user+" "+this.stage+"\t"+this.data);
    this.reset();
    this.main.setState(this.main.MSG_MAILDATA,this.ind);
  },
  reset : function(){
    this.mailCount=-1;
    this.mailData={};
    this.mailData.desc=this.getDesc();
    this.stage=(this.checkLogin?ST_CHECK:this.initStage);
    this.dataURLCopy=null;
    if(!this.checkLogin)this.cookieManager.clear();
  },
  checkMail : function(){
    if(!this.enabled)return;
    this.stop();
    //this.stage=this.mailCount<0?this.initStage:ST_DATA;
    if(this.mailCount<0)this.reset();
    else this.stage=ST_DATA;
    this.started=true;
    try{
      if(!this.process(null,""))++this.stage;
    }catch(e){
      this.onError();
dout(this.id+" "+e);
    }
  },
  onRequest : function(aHttpChannel,second){
    var cm=second?this.cookieManager2:this.cookieManager;
    var ck=cm.getCookie(aHttpChannel.URI,aHttpChannel);
    aHttpChannel.setRequestHeader("Cookie",ck, false);
  },
  onResponse : function(aHttpChannel,second){
    try {
      var cookie = aHttpChannel.getResponseHeader("Set-Cookie");
      var cm=second?this.cookieManager2:this.cookieManager;
      cm.addCookies(aHttpChannel.URI,cookie);
      aHttpChannel.setResponseHeader("Set-Cookie", "", false);
    }catch(e){}
  },
  getMailURL : function(aFolder){
    return this.mailURL;
  },
  getIconURL : function(){
    try{
      var url=this.mailURL.match(/(((\S+):\/\/([^/]+))(\S*\/)?)([^/]*)/);
      if(url)return url[2]+"/favicon.ico";
    }catch(e){}
    return null;
  },
  getIconPage : function(){
    return this.mailURL;
  },
  getCount : function(aData){
    return -1;
  },
  getData : function(aData){
    return {};
  },
  getDesc : function(){
    var n=this.calcCount();
    return n>0?n:"";
  },
  calcCount : function() {
    var aCount=this.mailCount;
    if(this.main&&this.main.resetCounter){
      if(aCount>=0){
        var count=this.loadCount();
        if(aCount>=count)aCount-=count;
        else{
          this.saveCount(aCount>0?aCount:0)
          aCount=0;
        }
      }
    }
    return aCount;
  },
  loadCount:function(){
    var prefStr="accounts."+this.id+"."+this.user+".count";
    var count=null;
    try{
      count=this.main.prefBranch.getIntPref(prefStr);
    }catch(e){}
    if(count==null){
      this.main.prefBranch.setIntPref(prefStr,0);
      count=0;
    }
    return count;
  },
  saveCount:function(n){
    var prefStr="accounts."+this.id+"."+this.user+".count";
    this.main.prefBranch.setIntPref(prefStr,n);
  },
  getForm:function(data,name){
    if(name){
      var reg=new RegExp("<form.+?name\\s*=\\s*[\"\']"+name+"[\"\']([\\S\\s]+?)<\/form>","i");
      var s=data.match(reg);
      if(!s)return "";
      data=s[1];
    }
    var re=/<input[^>]+?name\s*=\s*[\"\'](\S+?)[\"\'][^>]+?value\s*=s*[\"\']([\s\S]*?)[\"\']/ig;
    var o;
    var post="";
    while ((o = re.exec(data)) != null){
      if(post)post+="&";
      post+=o[1]+"="+encodeURIComponent(o[2]);
    }
    return post;
  },
  delay:function(sec){
    var timer=Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    timer.initWithCallback(this,sec,Components.interfaces.nsITimer.TYPE_ONE_SHOT);    
  },
  notify:function(aTimer){
    if(!this.process(null,""))++this.stage;
  }
};
WebMailHandler.prototype.baseProcess=WebMailHandler.prototype.process
/***********************************************************
load html
***********************************************************/
WebMailHandler.prototype.getHtml = function(aURL,aPostData,aHeaders,aMethod) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
  var uri = ioService.newURI(aURL, null, null);
  var channel = ioService.newChannelFromURI(uri);
  var httpChannel = channel.QueryInterface(Ci.nsIHttpChannel);
  if (aPostData||aPostData=="") {
    var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
                        .createInstance(Ci.nsIStringInputStream);
    uploadStream.setData(aPostData, aPostData.length);
    var uploadChannel = channel.QueryInterface(Ci.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
    httpChannel.requestMethod = "POST";
  }
  if(aHeaders){
    for(var t in aHeaders){
      httpChannel.setRequestHeader(t,aHeaders[t],false);
    }
  }
  if(aMethod)httpChannel.requestMethod=aMethod;
  this.channel=channel;
  channel.notificationCallbacks = this;
  channel.asyncOpen(this,httpChannel);
}
WebMailHandler.prototype.getCookie = function(domain,name){
  var obj = Components.classes["@mozilla.org/cookiemanager;1"].
              getService(Ci.nsICookieManager2);
  var enm = obj.enumerator;
  while(enm.hasMoreElements()){
    var ck = enm.getNext();
    ck.QueryInterface(Ci.nsICookie2);
    if(endsWith(ck.host,domain)&&ck.name==name)return ck;
  }
  return null;
};
WebMailHandler.prototype.getCookieString = function(domain,name){
  var ck=this.getCookie(domain,name);
  if(!ck)return null;
  var s=ck.name+"="+ck.value;
  if(ck.expires!=0)s+="; expires="+new Date(ck.expires*1000).toUTCString();
  if(ck.path)s+="; path="+ck.path;
  if(ck.host)s+="; domain="+ck.host;
  if(ck.secure)s+="; secure";
  if(ck.httponly)s+="; httponly"; 
  return s;
};

function encodeArray(data){
  return data;
}
////////////////for debug//////////////////////////////////
function dlog(str){
  dout(str);
  var name="wmn-log.txt";
  if(enableDebug==2)this.notFirst=true;
  str=new Date()+"\t"+str.replace(/\r/g,"\\r").replace(/\n/g,"\\n")+"\r\n";
  WebMailNotifier.prototype.saveFile(name,str,this.notFirst);
  this.notFirst=true;
}
///////////////////////////////////////////////////////////
